require 'helper'

describe Acting do
  Positivist = Module.new do
    def speak
      'YES'
    end
  end

  Negativist = Module.new do
    def speak
      'NO'
    end
  end

  it 'lets the whole cast play for duration of a block' do
    model1 = new_model
    model2 = new_model 

    Acting.new(model1 => Positivist, model2 => Negativist).play do
      assert_equal 'YES', model1.speak
      assert_equal 'NO',  model2.speak
    end

    refute model1.respond_to?(:speak)
    refute model2.respond_to?(:speak)
  end

  it 'can be manually startet/stopped by omitting a block' do
    model1 = new_model
    model2 = new_model 

    acting = Acting.new(model1 => Positivist, model2 => Negativist)

    refute model1.respond_to?(:speak)
    refute model2.respond_to?(:speak)

    acting.play

    assert_equal 'YES', model1.speak
    assert_equal 'NO',  model2.speak

    acting.quit

    refute model1.respond_to?(:speak)
    refute model2.respond_to?(:speak)
  end

  it 'fails when actor is no cast' do
    model = Object.new

    error = assert_raises RuntimeError do
      Acting.new(model => Positivist)
    end

    assert_match(/kind of Acting::Cast/, error.message)
  end

  it 'does not affect method_missing' do
    cls = Class.new do
      include Acting::Cast
      attr_reader :test_called

      def method_missing(meth, *args, &blk)
        if meth == :test
          @test_called = :true
        else
          super
        end
      end
    end

    model = cls.new

    Acting.new(model => Positivist).play do
      model.test
      assert_equal 'YES', model.speak
    end

    assert model.test_called
  end

  it 'allows stacking of actings' do
    model = new_model

    Acting.new(model => Positivist).play do
      assert_equal 'YES', model.speak

      Acting.new(model => Positivist).play do
        assert_equal 'YES', model.speak
      end

      assert_equal 'YES', model.speak
    end

    refute model.respond_to?(:speak)
  end

  private
  def new_model
    model = Object.new
    model.extend Acting::Cast
    model
  end
end
